@@ -28,7 +28,9 @@ my $stuff = {
     side => "Captain Planet!",
 };
 
-my $count = 2000000;
+my $count = 400000;
+
+print "Comparing hash of ASCII strings...\n";
 
 cmpthese (
     $count,
@@ -72,8 +74,15 @@ my $h2n = {
     x => 8_388_608,
     y => 16_777_216,
     z => 33_554_432,
+    A => 67_108_864,
+    B => 134_217_728,
+    C => 268_435_456,
+    D => 536_870_912,
+    E => 1_073_741_824,
 };
 
+print "Comparing hash of integers...\n";
+
 cmpthese (
     $count,
     {
@@ -89,3 +98,34 @@ cmpthese (
     },    
 );
 
+use utf8;
+
+my %unihash = (
+    'う' => '雨',
+    'あ' => '亜',
+    'い' => '井',
+    'え' => '絵',
+    'お' => '尾',
+    'ば' => [
+	qw/場 馬 羽 葉 刃/
+    ],
+);
+
+
+print "Comparing hash of Unicode strings...\n";
+
+cmpthese (
+    $count,
+    {
+	'JC' => sub {
+	    my $x = JSON::Create::create_json (\%unihash);
+	},
+	'JX' => sub {
+	    my $x = JSON::XS::encode_json (\%unihash);
+	},
+	'CJX' => sub {
+	    my $x = Cpanel::JSON::XS::encode_json (\%unihash);
+	},
+    },    
+);
+
